package com.qmth.sop.server.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.sop.business.bean.params.DingAddParam;
import com.qmth.sop.business.bean.result.CanChooseSopResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.service.TBDingAddService;
import com.qmth.sop.common.annotation.OperationLog;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.LogTypeEnum;
import com.qmth.sop.common.util.Result;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * @Description: 工时增加Controller
 * @Author: CaoZixuan
 * @Date: 2024-11-05
 */
@Api(tags = "工时增加Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_DING_ADD)
public class TBDingAddController {

    @Resource
    private TBDingAddService tbDingAddService;

    @ApiOperation(value = "增加工时-查询可选sop")
    @RequestMapping(value = "/find_sop_list", method = RequestMethod.POST)
    @ApiResponses({ @ApiResponse(code = 200, message = "返回信息", response = CanChooseSopResult.class) })
    public Result findSopList(@ApiParam(value = "服务单元id", required = true) @RequestParam Long serviceId, @ApiParam(value = "待追加用户id", required = true) @RequestParam Long userId) {
        return ResultUtil.ok(tbDingAddService.findCanChooseSopResultList(serviceId, userId));
    }

    @ApiOperation(value = "增加工时-提交")
    @RequestMapping(value = "/submit", method = RequestMethod.POST)
    @ApiResponses({ @ApiResponse(code = 200, message = "更新成功", response = Result.class) })
    @OperationLog(logType = LogTypeEnum.ADD)
    public Result submit(@Valid @RequestBody DingAddParam dingAddParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        tbDingAddService.dingAddSubmit(dingAddParam, requestUser.getId());
        return ResultUtil.ok();
    }
}
