package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.params.DingAddParam;
import com.qmth.sop.business.bean.result.CanChooseSopResult;
import com.qmth.sop.business.entity.TBDingAdd;
import com.qmth.sop.business.mapper.TBDingAddMapper;
import com.qmth.sop.business.service.TBDingAddService;
import com.qmth.sop.business.service.TBUserArchivesService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 工时追加表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-11-04
 */
@Service
public class TBDingAddServiceImpl extends ServiceImpl<TBDingAddMapper, TBDingAdd> implements TBDingAddService {

    @Resource
    private TBUserArchivesService tbUserArchivesService;

    @Override
    public CanChooseSopResult findCanChooseSopResultList(Long serviceId, Long userId) {
        CanChooseSopResult result = new CanChooseSopResult();
        boolean containsRegionCoordinator = tbUserArchivesService.containsRegionCoordinator(userId);
        if (containsRegionCoordinator) {
            result.setNeedChoose(false);
        } else {
            result.setNeedChoose(true);
            result.setCrmDetailInfo(this.baseMapper.findSopInfoByServiceIdAndUserId(serviceId, userId));
        }
        return result;
    }

    @Transactional
    @Override
    public void dingAddSubmit(DingAddParam dingAddParam, Long requestUserId) {
        Long userId = dingAddParam.getUserId();
        Long serviceId = dingAddParam.getServiceId();
        String sopNo = dingAddParam.getSopNo();
        BigDecimal addDays = dingAddParam.getAddDays();
        BigDecimal addHours = dingAddParam.getAddHours();

        boolean isCoordinator = tbUserArchivesService.containsRegionCoordinator(userId);
        List<TBDingAdd> dbDingAddList = this.list(
                new QueryWrapper<TBDingAdd>().lambda().eq(TBDingAdd::getServiceId, serviceId).eq(TBDingAdd::getUserId, userId));

        QueryWrapper<TBDingAdd> removeQuery = new QueryWrapper<>();
        removeQuery.lambda().eq(TBDingAdd::getServiceId, serviceId).eq(TBDingAdd::getUserId, userId);

        if (isCoordinator) {
            // 区协追加
            // 如果有曾经作为工程师的追加记录，则不允许提交
            if (dbDingAddList.stream().anyMatch(e -> Objects.nonNull(e.getSopNo()))) {
                throw ExceptionResultEnum.ERROR.exception("该用户曾有过工程师的追加记录，无法作为区协追加");
            }
        } else {
            // 工程师追加
            // 工程师追加必须选择SOP
            if (Objects.isNull(sopNo)) {
                throw ExceptionResultEnum.ERROR.exception("工程师必须选择SOP");
            }
            // 如果有曾经作为区协的追加记录，则不允许提交
            if (dbDingAddList.stream().anyMatch(e -> Objects.isNull(e.getSopNo()))) {
                throw ExceptionResultEnum.ERROR.exception("该用户曾有过区协的追加记录，无法作为工程师追加");
            }
            removeQuery.lambda().eq(TBDingAdd::getSopNo, sopNo);
        }

        // 清除掉之前的追加记录
        this.remove(removeQuery);
        // 追加
        TBDingAdd dingAdd = new TBDingAdd();
        dingAdd.setServiceId(serviceId);
        dingAdd.setUserId(userId);
        if (Objects.nonNull(sopNo) && !isCoordinator) {
            dingAdd.setSopNo(sopNo);
        }
        if (SystemConstant.effectBigDecimal(addDays)) {
            dingAdd.setAddDays(addDays);
        } else {
            dingAdd.setAddHours(addHours);
        }
        dingAdd.insertInfo(requestUserId);
        if (SystemConstant.effectBigDecimal(addDays) || SystemConstant.effectBigDecimal(addHours)) {
            // 追加日或工时有效才追加 否则视为删除追加操作
            this.save(dingAdd);
        }
    }
}
