package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.DingStatisticBySupplierDto;
import com.qmth.sop.business.entity.TBDingAdd;
import com.qmth.sop.business.entity.TBDingStatistic;
import com.qmth.sop.business.entity.TBDingStatisticBySupplier;
import com.qmth.sop.business.entity.TBUserArchivesSupplier;
import com.qmth.sop.business.mapper.TBDingStatisticBySupplierMapper;
import com.qmth.sop.business.service.SysSupplierService;
import com.qmth.sop.business.service.SysUserService;
import com.qmth.sop.business.service.TBDingStatisticBySupplierService;
import com.qmth.sop.business.service.TBUserArchivesSupplierService;
import com.qmth.sop.common.contant.SystemConstant;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 工时统计-按供应商统计 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-09-09
 */
@Service
public class TBDingStatisticBySupplierServiceImpl extends ServiceImpl<TBDingStatisticBySupplierMapper, TBDingStatisticBySupplier> implements TBDingStatisticBySupplierService {

    @Resource
    SysSupplierService sysSupplierService;

    @Resource
    TBUserArchivesSupplierService tbUserArchivesSupplierService;

    @Resource
    SysUserService sysUserService;

    @Transactional
    @Override
    public void buildStatisticBySupplier(List<TBDingStatistic> datasource, List<TBDingAdd> dingAddDatasource) {
        List<TBDingStatisticBySupplier> result = new ArrayList<>();
        List<Long> serviceIdList = datasource.stream().map(TBDingStatistic::getServiceId).distinct().collect(Collectors.toList());
        for (Long serviceId : serviceIdList) {
            List<TBDingStatistic> serviceStatisticList = datasource.stream().filter(e -> serviceId.equals(e.getServiceId())).collect(Collectors.toList());
            List<Long> supplierIdList = serviceStatisticList.stream().map(TBDingStatistic::getSupplierId).distinct().collect(Collectors.toList());

            for (Long supplierId : supplierIdList) {
                List<TBDingStatistic> tbDingStatisticList = serviceStatisticList.stream().filter(e -> supplierId.equals(e.getSupplierId())).collect(Collectors.toList());
                String serviceName = "";
                String supplierName = sysSupplierService.getById(supplierId).getName();
                int actualDays = 0;
                int weekdays = 0;
                int weekends = 0;
                int legalHolidays = 0;
                BigDecimal workHours = new BigDecimal(0);
                BigDecimal addDays = new BigDecimal(0);
                BigDecimal addHours = new BigDecimal(0);
                int violationDays = 0;

                for (TBDingStatistic tbDingStatistic : tbDingStatisticList) {
                    serviceName = tbDingStatistic.getServiceName();
                    actualDays = actualDays + tbDingStatistic.getActualDays();
                    weekdays = weekdays + tbDingStatistic.getWeekdays();
                    weekends = weekends + tbDingStatistic.getWeekends();
                    legalHolidays = legalHolidays + tbDingStatistic.getLegalHolidays();
                    workHours = workHours.add(tbDingStatistic.getWorkHours());
                    violationDays = violationDays + tbDingStatistic.getViolationDays();
                }
                TBDingStatisticBySupplier tbDingStatisticBySupplier = new TBDingStatisticBySupplier();
                tbDingStatisticBySupplier.setServiceId(serviceId);
                tbDingStatisticBySupplier.setServiceName(serviceName);
                tbDingStatisticBySupplier.setSupplierId(supplierId);
                tbDingStatisticBySupplier.setSupplierName(supplierName);
                tbDingStatisticBySupplier.setActualDays(actualDays);
                tbDingStatisticBySupplier.setWeekdays(weekdays);
                tbDingStatisticBySupplier.setWeekends(weekends);
                tbDingStatisticBySupplier.setLegalHolidays(legalHolidays);
                tbDingStatisticBySupplier.setWorkDays(BigDecimal.valueOf(actualDays));
                tbDingStatisticBySupplier.setWorkHours(workHours);
                tbDingStatisticBySupplier.setViolationDays(violationDays);

                // 处理工时追加
                List<Long> userIdList = this.findUserIdListBySupplierId(supplierId);
                List<TBDingAdd> dingAddList = dingAddDatasource.stream().filter(e -> serviceId.equals(e.getServiceId()) && userIdList.contains(e.getUserId()))
                        .collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(dingAddList)) {
                    for (TBDingAdd tbDingAdd : dingAddList) {
                        if (SystemConstant.effectBigDecimal(tbDingAdd.getAddDays())) {
                            addDays = addDays.add(tbDingAdd.getAddDays());
                        }
                        if (SystemConstant.effectBigDecimal(tbDingAdd.getAddHours())) {
                            addHours = addHours.add(tbDingAdd.getAddHours());
                        }
                    }
                    if (SystemConstant.effectBigDecimal(addDays)) {
                        tbDingStatisticBySupplier.setAddDays(addDays);
                    }
                    if (SystemConstant.effectBigDecimal(addHours)) {
                        tbDingStatisticBySupplier.setAddHours(addHours);
                    }
                    tbDingStatisticBySupplier.setWorkDays(tbDingStatisticBySupplier.getWorkDays().add(addDays));
                    tbDingStatisticBySupplier.setWorkHours(workHours.add(addHours));
                }
                result.add(tbDingStatisticBySupplier);
            }
        }
        this.baseMapper.truncateTable();
        this.saveBatch(result);
    }

    @Override
    public IPage<DingStatisticBySupplierDto> findStatisticBySupplierPage(Long serviceId, Integer pageNumber,
            Integer pageSize) {
        return this.baseMapper.findStatisticBySupplierPage(new Page<>(pageNumber, pageSize), serviceId);
    }

    @Override
    public List<DingStatisticBySupplierDto> findStatisticBySupplierList(Long serviceId) {
        return this.baseMapper.findStatisticBySupplierList(serviceId);
    }

    /**
     * 根据供应商查询该供应商所有的用户id
     *
     * @param supplierId 供应商id
     * @return 用户id集合
     */
    private List<Long> findUserIdListBySupplierId(Long supplierId) {
        List<Long> tbUserArchivesIdList = tbUserArchivesSupplierService.list(
                        new QueryWrapper<TBUserArchivesSupplier>().lambda().eq(TBUserArchivesSupplier::getSupplierId, supplierId)).stream()
                .map(TBUserArchivesSupplier::getUserArchivesId).distinct().collect(Collectors.toList());
        return tbUserArchivesIdList.stream().flatMap(e -> Stream.of(sysUserService.findByArchivesId(e).getId())).collect(Collectors.toList());
    }
}